@@ -27,6 +27,7 @@ from telepathy.constants import (CONNECTION_HANDLE_TYPE_NONE,
 from telepathy.errors import InvalidArgument
 
 from telepathy.interfaces import (CHANNEL_INTERFACE,
+                                  CHANNEL_INTERFACE_CONFERENCE,
                                   CHANNEL_INTERFACE_DTMF,
                                   CHANNEL_INTERFACE_GROUP,
                                   CHANNEL_INTERFACE_HOLD,
@@ -64,8 +65,6 @@ class Channel(_Channel, DBusProperties):
         self._type = props[CHANNEL_INTERFACE + '.ChannelType']
         self._requested = props[CHANNEL_INTERFACE + '.Requested']
 
-        self._immutable_properties = dict()
-
         tht = props.get(CHANNEL_INTERFACE + '.TargetHandleType', HANDLE_TYPE_NONE)
 
         if tht == HANDLE_TYPE_NONE:
@@ -86,7 +85,7 @@ class Channel(_Channel, DBusProperties):
              'TargetID': lambda: dbus.String(self._handle.get_name()),
              'Requested': lambda: self._requested})
 
-        self._add_immutables({
+        self._add_immutable_properties({
             'ChannelType': CHANNEL_INTERFACE,
             'TargetHandle': CHANNEL_INTERFACE,
             'Interfaces': CHANNEL_INTERFACE,
@@ -96,22 +95,18 @@ class Channel(_Channel, DBusProperties):
             })
 
     def _add_immutables(self, props):
-        self._immutable_properties.update(props)
-
-    def get_props(self):
-        """Despite its name, this function actually only returns immutable channel
-        properties."""
-        props = dict()
-        for prop, iface in self._immutable_properties.items():
-            props[iface + '.' + prop] = \
-                self._prop_getters[iface][prop]()
-        return props
+        #backward compatibility
+        self._add_immutable_properties(props)
 
     @dbus.service.method(CHANNEL_INTERFACE, in_signature='', out_signature='')
     def Close(self):
         self.Closed()
-        self._chan_manager.remove_channel(self)
-        self._conn.remove_channel(self)
+        try:
+            self._chan_manager.remove_channel(self)
+            self._conn.remove_channel(self)
+            self.remove_from_connection()
+        except:
+            pass
 
     @dbus.service.method(CHANNEL_INTERFACE, in_signature='', out_signature='s')
     def GetChannelType(self):
@@ -151,6 +146,9 @@ class ChannelTypeContactList(Channel, _ChannelTypeContactListIface):
         Channel.__init__(self, connection, manager, props,
             object_path=object_path)
 
+    def Close(self):
+        raise telepathy.NotImplemented("Contact lists can't be closed")
+
 
 from telepathy._generated.Channel_Type_File_Transfer \
         import ChannelTypeFileTransfer as _ChannelTypeFileTransferIface
@@ -204,7 +202,7 @@ class ChannelTypeRoomList(Channel, _ChannelTypeRoomListIface):
         self._listing_rooms = False
         self._rooms = {}
 
-        self._add_immutables({'Server': CHANNEL_TYPE_ROOM_LIST})
+        self._add_immutable_properties({'Server': CHANNEL_TYPE_ROOM_LIST})
 
     @dbus.service.method(CHANNEL_TYPE_ROOM_LIST, in_signature='', out_signature='b')
     def GetListingRooms(self):
@@ -303,6 +301,49 @@ class ChannelTypeText(Channel, _ChannelTypeTextIface):
 from telepathy._generated.Channel_Interface_Chat_State \
         import ChannelInterfaceChatState
 
+from telepathy._generated.Channel_Interface_Conference \
+        import ChannelInterfaceConference as _ChannelInterfaceConference
+
+class ChannelInterfaceConference(_ChannelInterfaceConference):
+
+    def __init__(self):
+        _ChannelInterfaceConference.__init__(self)
+
+        self._conference_channels = set()
+        self._conference_initial_channels = set()
+        self._conference_initial_invitees = set()
+        self._conference_invitation_message = ""
+        self._conference_original_channels = {}
+
+        # D-Bus properties for conference interface
+        self._implement_property_get(CHANNEL_INTERFACE_CONFERENCE, {
+            'Channels':
+                lambda: dbus.Array(self._conference_channels, signature='o'),
+            'InitialChannels':
+                lambda: dbus.Array(self._conference_initial_channels,
+                                   signature='o'),
+            'InitialInviteeHandles':
+                lambda: dbus.Array(
+                    [h.get_id() for h in self._conference_initial_invitees],
+                    signature='u'),
+            'InitialInviteeIDs':
+                lambda: dbus.Array(
+                    [h.get_name() for h in self._conference_initial_invitees],
+                    signature='s'),
+            'InvitationMessage':
+                lambda: dbus.String(self._conference_invitation_message),
+            'OriginalChannels':
+                lambda: dbus.Dictionary(self._conference_original_channels,
+                                        signature='uo')
+            })
+
+        # Immutable conference properties
+        self._add_immutable_properties({
+                'InitialChannels': CHANNEL_INTERFACE_CONFERENCE,
+                'InitialInviteeIDs': CHANNEL_INTERFACE_CONFERENCE,
+                'InitialInviteeHandles': CHANNEL_INTERFACE_CONFERENCE,
+                'InvitationMessage': CHANNEL_INTERFACE_CONFERENCE
+                })
 
 from telepathy._generated.Channel_Interface_DTMF import ChannelInterfaceDTMF
 
@@ -310,11 +351,10 @@ from telepathy._generated.Channel_Interface_DTMF import ChannelInterfaceDTMF
 from telepathy._generated.Channel_Interface_Group \
         import ChannelInterfaceGroup as _ChannelInterfaceGroup
 
-class ChannelInterfaceGroup(_ChannelInterfaceGroup, DBusProperties):
+class ChannelInterfaceGroup(_ChannelInterfaceGroup):
 
     def __init__(self):
         _ChannelInterfaceGroup.__init__(self)
-        DBusProperties.__init__(self)
 
         self._implement_property_get(CHANNEL_INTERFACE_GROUP,
             {'GroupFlags': lambda: dbus.UInt32(self.GetGroupFlags()),
