@@ -21,12 +21,13 @@ import dbus
 import dbus.service
 
 from telepathy.errors import NotImplemented
-from telepathy.interfaces import CONN_MGR_INTERFACE
+from telepathy.interfaces import (CONN_MGR_INTERFACE)
+from telepathy.server.properties import DBusProperties
 
 from telepathy._generated.Connection_Manager \
         import ConnectionManager as _ConnectionManager
 
-class ConnectionManager(_ConnectionManager):
+class ConnectionManager(_ConnectionManager, DBusProperties):
     def __init__(self, name):
         """
         Initialise the connection manager.
@@ -37,8 +38,17 @@ class ConnectionManager(_ConnectionManager):
                                     dbus.service.BusName(bus_name, dbus.Bus(), do_not_queue=True),
                                     object_path)
 
+        self._interfaces = set()
         self._connections = set()
-        self._protos = {}
+        self._protos = {} # proto name => Connection constructor
+        self._protocols = {} # proto name => Protocol object
+
+        DBusProperties.__init__(self)
+        self._implement_property_get(CONN_MGR_INTERFACE, {
+                'Interfaces': lambda: dbus.Array(self._interfaces, signature='s'),
+                'Protocols': lambda: dbus.Dictionary(self._protocol_properties,
+                                                     signature='sa{sv}')
+                })
 
     def connected(self, conn):
         """
@@ -60,14 +70,41 @@ class ConnectionManager(_ConnectionManager):
 
         return False # when called in an idle callback
 
+    def check_proto(self, proto):
+        if proto not in self._protos:
+            raise NotImplemented('unknown protocol %s' % proto)
+
+    def check_protocol(self, proto):
+        if proto not in self._protocols:
+            raise NotImplemented('no protocol object for %s' % proto)
+
+    @dbus.service.method(CONN_MGR_INTERFACE, in_signature='s', out_signature='a(susv)')
+    def GetParameters(self, proto):
+        "Returns the mandatory and optional parameters for the given proto."
+        self.check_proto(proto)
+        self.check_protocol(proto)
+
+        return self._protocols[proto].parameters
+
     @dbus.service.method(CONN_MGR_INTERFACE, in_signature='', out_signature='as')
     def ListProtocols(self):
         return self._protos.keys()
 
     def RequestConnection(self, proto, parameters):
-        if proto in self._protos:
-            conn = self._protos[proto](self, parameters)
-            self.connected(conn)
-            return (conn._name.get_name(), conn._object_path)
-        else:
-            raise NotImplemented('unknown protocol %s' % proto)
+        self.check_proto(proto)
+
+        conn = self._protos[proto](self, parameters)
+        self.connected(conn)
+        return (conn._name.get_name(), conn._object_path)
+
+    def _implement_protocol(self, name, protocol_class):
+        protocol = protocol_class(self)
+        self._protocols[name] = protocol
+        self._protos[name] = protocol.create_connection
+
+    @property
+    def _protocol_properties(self):
+        properties = {}
+        for name, protocol in self._protocols.items():
+            properties[name] = protocol.get_immutable_properties()
+        return properties
